#ifndef HEXAPOD_ROBOT_HWI_
#define HEXAPOD_ROBOT_HWI_

#include <memory>
#include <string>
#include <vector>
#include <unordered_set>
#include <map>

#include "hardware_interface/handle.hpp"
#include "hardware_interface/hardware_info.hpp"
#include "hardware_interface/system_interface.hpp"
#include "hardware_interface/types/hardware_interface_return_values.hpp"
#include "rclcpp/macros.hpp"
#include "rclcpp_lifecycle/node_interfaces/lifecycle_node_interface.hpp"
#include "rclcpp_lifecycle/state.hpp"

namespace hexapod_robot {

struct PWMOutput {
    uint8_t i2c_address;
    uint8_t channel;
};

class HexapodRobotHardwareInterface : public hardware_interface::SystemInterface {

public:
    RCLCPP_SHARED_PTR_DEFINITIONS(HexapodRobotHardwareInterface)
    
    hardware_interface::CallbackReturn on_init(const hardware_interface::HardwareInfo &info) override;
    
    std::vector<hardware_interface::StateInterface> export_state_interfaces() override;
    std::vector<hardware_interface::CommandInterface> export_command_interfaces() override;
    
    hardware_interface::CallbackReturn on_activate(const rclcpp_lifecycle::State &previous_state) override;
    hardware_interface::CallbackReturn on_deactivate(const rclcpp_lifecycle::State &previous_state) override;
    
    hardware_interface::return_type read(const rclcpp::Time & time, const rclcpp::Duration & period) override;
    hardware_interface::return_type write(const rclcpp::Time & time, const rclcpp::Duration & period) override;

private:
    int i2c_fd_;
    bool i2c_available_;

    std::map<uint8_t, bool> pca9685_controller_available_;

    std::map<std::string, double> joint_states_;
    std::map<std::string, double> joint_commands_;

    std::map<std::string, PWMOutput> joint_pwm_mapping_;

};

} // namespace

#endif // HEXAPOD_ROBOT_HWI_
